
(* :Title: Cartesian$Products *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
This package solves exercises with Cartesian products sets.
*)

(* :Context: SetTheory`Cartesian$Products` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1999. Adapted for MSP, 2002.
*)

(* :Keywords:
	SetTheory, CartesianProducts
*)

(* :Source: None. *)

(* :Warning: *)

(* :Mathematica Version: 3.0 or higher *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)

BeginPackage["SetTheory`Cartesian$Products`"]

MSP$cp$1::usage =
"MSP$cp$1[R] picks up the messages from cp$1[R]."

cp$1::usage =
"cp$1[R] analyses the answer R."

MSP$Syntax$Check::usage =
"MSP$Syntax$Check[expr] is used to catch syntax error messages from syntax$check."

syntax$check::usage =
"syntax$check[expr] returns True if expr is a valid expression, and False ohterwise."


CartesianProducts$msg

SyntaxCheck$msg


Unprotect[cp$1, MSP$cp$1, syntax$check, MSP$Syntax$Check]

Begin["`Private`"]

cpnprotected$1 = Unprotect[randomsets, testanswer, wronganswer,
                           cartesian$product, f$1, cp2sets, cpnsets,
                           stepbystep, select$exp, convert$output,
                           convert$input]

cellbkg = RGBColor[1.,0.380001,0.009995] (* CadmiumOrange *) 


(* Auxiliary Functions *)

expressions$list = {"A\[Times]B", "A\[Times]A", 
                    "A\[Times]A\[Times]A", "A\[Times](B\[Times]C)",
                    "(A\[Times]B)\[Times]C"}

X$sets = {{"1", "2", "3", "4"}, {"a", "b", "c", "d"}, 
          {"x", "y", "z", "w"}}

select$exp[x___] := Module[{sel$exp},
     cnt$ans = 0;
     exp$flag = 0;
     n$exp = Random[Integer, {1,Length[expressions$list]}];
     sel$exp = expressions$list[[n$exp]];
     n$sets = Length[Union[Characters[
              StringReplace[sel$exp, {"("->"",")"->"","\[Times]"->""}]]]];
     n$parts = Length[Union[
               Select[Characters[sel$exp], (MemberQ[{"(",")"}, #])&]]];
     If[n$parts != 0, If[First[Characters[sel$exp]]=="(", exp$flag = 1,
                                                          exp$flag = 2 ]];  
     sel$exp
]

randomsets[n_] := Module[{l,i,out$sets},
     cnt$ans = 0;
     l    = Table[Random[Integer, {2,3}], {i,1,3}];
     sets = MapThread[Take[#1, #2]&, {X$sets, l}];
out$sets=StringDrop[StringJoin[Map[
             (FromCharacterCode[#+64]<>" = "<>ToString[sets[[#]]]<>"\n")&,
              Range[1,n]]], -1]
]

testanswer[answer__, res__] := answer === res

wronganswer[answer__, res__] := Module[{},
     test = Intersection[answer, res];
     test$out   = convert$output[ToString[test]];
     answer$out = convert$output[ToString[answer]];

     test = Intersection[answer, res];
     If[ Length[test]==0,
       text = "Your answer: "<>
             ToString[SetTheory`Cartesian$Products`Private`answer$out]<>
             " is incorrect.",
       text = "Your answer: "<>
             ToString[SetTheory`Cartesian$Products`Private`answer$out]<>
             " is partially correct.\n"<>
             "The correct part is: "<>
             ToString[SetTheory`Cartesian$Products`Private`test$out]
       ];
text
]


cartesian$product[{x__}]:=Module[{n,s},
		n=Length[{x}];
		If[n==0,Return[{}]];
		s=ToString[Map[("#"<>ToString[#])&,Range[1,n]]]<>"&" ;
		Flatten[Outer[ToExpression[s],x],n-1]
	]


f$1[x_,y_List]:=Map[{x,#}&,y]

cp2sets[{x_List,y_List}]:= Which[Length[x]==0,y,Length[y]==0,x,
	                         True, Flatten[Map[f$1[#,y]&,x],1]
]

stepbystep[x___]:=Module[{list1,lists,tmp},
		If[Length[x]==0, Return[product$cp]];
		{list1,lists}={First[x],Rest[x]};
		tmp=cp2sets[{list1,product$cp}];
		product$cp=tmp;
		stepbystep[lists]
]

cpnsets[{x___}]:=Module[{},
		If[Length[{x}]==0,Return[{}]];
		product$cp={};
		stepbystep[Reverse[{x}]]
]

convert$output[x_String] := "{"<>StringReplace[
                                 StringDrop[StringDrop[x, 1], -1], 
                                             {"{"->"(","}"->")"}]<>"}"

convert$input[x_String] := StringReplace[x, {"("->"{",")"->"}"}]



MSP$Syntax$Check[s_String] := Module[{tmp,res},
  tmp = Catch[ syntax$check[s], _SyntaxCheck$msg, List];
  res = tmp
]


syntax$check[s_String] := Module[{s0, s1, nleftb, nrightb, elements, sx},

string$check = False ;
s0    = StringReplace[s, {"{"->"", " "->"", "}"->""}];

If[StringLength[s0] > 0,
     s1 = Characters[s0];

     nleftb  = Length[Select[s1,(#=="(")&]];
     nrightb = Length[Select[s1,(#==")")&]];

     If[ nleftb != nrightb,
          Throw[255, SyntaxCheck$msg[ "Unbalanced Parenthesis"]];
          Return[False]];

     elements = Union[Map[ToString[#]&, Flatten[X$sets]], {"(",")",","," "}];
     sx       = Complement[s1, elements];
     sx$out   = ToString[sx];

     If[ Length[sx] != 0,
          Throw[255, SyntaxCheck$msg[ "Wrong Symbol(s) or Separator(s): \n"<>
	 				SetTheory`Cartesian$Products`Private`sx$out]];
	  Return[False]],

     Throw[255, SyntaxCheck$msg[ "No Input"]];
     Return[False]
];

string$check = True
]



(* Main Function *)

MSP$cp$1[R_String] := Module[{tmp,res},
  tmp = Catch[ syntax$check[R], _SyntaxCheck$msg, List];
  res = tmp;
  If[ tmp[[1]] == 255, Return[res] ];
  tmp = Catch[ cp$1[R], _CartesianProducts$msg, List];
  res = tmp
]


cp$1[R_String] := Module[{sel$sets, cp$sets, wrans, addcell, input},
     If[ cnt$ans>2,
         Throw[cnt$ans, CartesianProducts$msg[ "Close this window and try another exercise"<>
                                     " (i.e., generate\nnew sets and/or products)."]];
	 Return[False]
       ];

     sel$sets = Characters[
                StringReplace[expressions$list[[n$exp]],
                                    {"("->"",")"->"","\[Times]"->""}]];
     cp$sets  = sel$sets /. 
                {"A"->"SetTheory`Cartesian$Products`Private`sets[[1]]",     
                 "B"->"SetTheory`Cartesian$Products`Private`sets[[2]]",     
                 "C"->"SetTheory`Cartesian$Products`Private`sets[[3]]"};

     input    = ToExpression[convert$input[R]];
     answer   = input;

If[ exp$flag == 0,    
     res      = cartesian$product[ToExpression[cp$sets]]
];

If[ exp$flag == 1,
     res      = Map[{Take[#,2], Last[#]}&,
                cartesian$product[ToExpression[cp$sets]]]
];

If[ exp$flag == 2,
     res      = Map[{First[#], Rest[#]}&,
                cartesian$product[ToExpression[cp$sets]]]
];

     answer$1 = convert$output[ToString[answer]];
     res$1    = convert$output[ToString[res]];

     If[ testanswer[answer, res],
(* Right answer *)
       Throw[cnt$ans, PowerSets$msg[ "Your answer: "<>
                   ToString[SetTheory`Cartesian$Products`Private`answer$1]<>
                   " is correct."]];
       Return[True], 
(* Wrong answer *)
     wrans = wronganswer[answer, res];
     If[ cnt$ans==2,
         addcell = "You failed three times.\n"<>
                   "The correct answer is: "<> 
                   ToString[SetTheory`Cartesian$Products`Private`res$1];
         cnt$ans++,
         addcell = "Try again"; 
         cnt$ans++
         ]
       ];
     Throw[cnt$ans-1, CartesianProducts$msg[ wrans<>"\n\n"<>addcell]];
     Return[False]; 
]


(* EPILOG *)

SetAttributes[ randomsets,        ReadProtected]
SetAttributes[ testanswer,        ReadProtected]
SetAttributes[ wronganswer,       ReadProtected]
SetAttributes[ cartesian$product, ReadProtected]
SetAttributes[ f$1,               ReadProtected]
SetAttributes[ cp2sets,           ReadProtected]
SetAttributes[ cpnsets,           ReadProtected]
SetAttributes[ stepbystep,        ReadProtected]
SetAttributes[ select$exp,        ReadProtected]
SetAttributes[ convert$output,    ReadProtected]
SetAttributes[ convert$input,     ReadProtected]

Protect[Evaluate[cpnprotected$1]]

End[]  (*Cartesian$Products`Private`*)

SetAttributes[ cp$1,             ReadProtected]
SetAttributes[ MSP$cp$1,         ReadProtected]
SetAttributes[ syntax$check,     ReadProtected]
SetAttributes[ MSP$Syntax$Check, ReadProtected]

Protect[cp$1, MSP$cp$1, syntax$check, MSP$Syntax$Check]

EndPackage[] (* Cartesian$Products`*)


