
(* :Title: Mappings *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
This package displays a mapping between two sets and shows the
type (one-to-one, onto etc.) of the mapping.
*)

(* :Context: SetTheory`Mappings` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1999. 
Adapted for MSP 2002.
*)

(* :Keywords:
	SetTheory, Mapping
*)

(* :Source: None. *)

(* :Warning: *)

(* :Mathematica Version: 3.0 *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)
                  
BeginPackage["SetTheory`Mappings`"]

Mappings::usage = 
"Mappings[] displays and analyzes mappings bewteen two finite sets."

Mappings$Error

MSP$pairstring

m$protected = Unprotect[Mappings,Mappings$Error,MSP$pairstring]

Begin["`Private`"]

mappingsprotected = Unprotect[arrow,separateleft,separateright,maindiagram,
                             a,b,c,d,e,f,g,t,u,v,w,x,y,z,chars2coords,
                             pairstring,getfunctiontype,move]

(* Colors *)

Green = RGBColor[0., 1.0, 0.]

cellbkg = RGBColor[1.,0.380001,0.009995] (* CadmiumOrange *) 

Red = RGBColor[1.0, 0., 0.];

ftype = RGBColor[0.529405,0.807794,0.980407];

(* Parameter and Function Definitions *)

getfunctiontype[mask_]:=Module[{types,indexes,typestrings,typeoutstring},
types = {"a RELATION not a","a 1-1","an ONTO"," a 1-1 and ONTO",
         "a GENERAL"};
typeoutstring = types[[mask]]
];

numbers = {"0","1","2","3","4","5","6","7","8","9"};

nolf = { "\\012"->"", " "->"", "("->"{", ")"->"}" } ;

(*
leftsymbols={"a","b","c","d","e","f","g"};

rightsymbols={"t","u","v","w","x","y","z"};
*)

pairstring[arg_] := Module[
		{nl,nr,s0,s1,s2,sl,sr,s3,sx,s11,nlb,nrb,ncommas,nsymbols},

  If[Length[arg]!=3,
     Throw[False, Mappings$Error["3 arguments expected."]],
(*     Return[maindiagram[{},opts]], *)
     leftsymbols=Characters[StringReplace[ToString[arg[[1]]],
                            {"("->"",")"->"","{"->"",","->"","}"->""," "->""}]];
     rightsymbols=Characters[StringReplace[ToString[arg[[2]]],
                            {"("->"",")"->"","{"->"",","->"","}"->""," "->""}]];
     nl = Length[leftsymbols];
     nr = Length[rightsymbols];
     If[!NumberQ[nl],
        Throw[False, Mappings$Error["nl -> not a number."]]];
(*     Return[maindiagram[{},opts]]]; *)
     If[!NumberQ[nr], 
        Throw[False, Mappings$Error["nr -> not a number."]]];
(*     Return[maindiagram[{},opts]]]; *)
     If[!(0<nl<8),
        Throw[False, Mappings$Error["Integer in the range [1,7] expected."]]];
(*     Return[maindiagram[{},opts]]]; *)
     If[!(0<nr<8), 
        Throw[False, Mappings$Error["Integer in the range [1,7] expected."]]];
(*     Return[maindiagram[{},opts]]] *)
    ];


        relation$mask = False;
        s0 = arg[[3]];
	s1 = StringReplace[s0, nolf];

        s11 = StringReplace[s1, {"{"->"","}"->""}];
	sx  = Characters[s1];
	nlb = Length[Select[sx, (#=="{")&]];
	nrb = Length[Select[sx, (#=="}")&]];
	If[nlb != nrb,
           Throw[False, Mappings$Error["Unbalanced curly braces."]]];

	ncommas = Length[Select[sx, (#==",")&]];
	If[((Length[sx]-4)/3) != ncommas, 
           Throw[False, Mappings$Error["Too many commas or missing comma(s)."]]];

	s2 = Partition[
		Characters[StringReplace[s1,{"{"->"","}"->"",","->""}]],2];
	{sl,sr} = Transpose[s2];
	s3 = Length /@ Map[Union[#]&, s2];
	If[Length[Select[s3, #!=2&]] != 0, 
           Throw[False, Mappings$Error["Identical elements in one or more pairs."]]];

elements=Flatten[
         {Take[leftsymbols,nl],Take[rightsymbols,nr],"{","}",","}];

	If[Length[Select[Characters[s1], !MemberQ[elements,#]&]] != 0,
           Throw[False, Mappings$Error[s1<>"\nYou may use only the symbols from the diagram."]]];

	nsymbols = (Length[sx]-1)/3;
	If[Length[Select[sx, MemberQ[Drop[elements,-3],#]&]] != nsymbols, 
           Throw[False, Mappings$Error[s1<>"\nToo many symbols or missing symbol."]]];

	If[Length[Intersection[sl,Take[rightsymbols,nr]]] != 0,
           Throw[False, Mappings$Error[s1<>"\nRelation(s) defined inside the codomain or\n"<>
                "inverse relation(s) defined."]]];

	If[Length[Intersection[sr,Take[leftsymbols,nl]]] != 0, 
           Throw[False, Mappings$Error[s1<>"\nRelation(s) defined inside the domain."]]];
        If[Length[Intersection[sl,Take[leftsymbols,nl]]] != nl,
(*		CellPrint[Cell[s1<>
		"\nYou have to use all the elements from the domain.", "Text",
                CellFrame->True, Background->cellbkg]];
     SelectionMove[EvaluationNotebook[],Before,Cell]; *)
                relation$mask = True
	];
{True, s1}
];


MSP$pairstring[args_] := Module[{tmp,res},
  tmp = Catch[ pairstring[args], _Mappings$Error, List ];
  res = tmp
]


maindiagram[x__, opts___]:=
		Show[Graphics[{Green,PointSize[0.03],points}],
		Axes->False,
		Frame->True, FrameTicks->{None,None},
		AspectRatio->1,
		PlotRange->{{-1,1},{-1,1}},
		Prolog->labels, 
		Epilog->{x}, opts];

move[{x_,y_},alpha_,{a_,b_}]:= {x*Cos[alpha]+y*Sin[alpha]+a,
                               -x*Sin[alpha]+y*Cos[alpha]+b};

arrow[{{x1_,y1_},{x2_,y2_}},color_] := Module[
{arrow0,beta,l,yb,xb,length,alpha,arrowhead,shorterl,x2p,y2p},

     l      = 0.05 * 4.0 / 3.0;
     beta   = Pi/18;
     yb     = l*Sin[beta];
     xb     = l*Cos[beta];
     arrow0 = {{-xb,yb},{0,0},{-xb,-yb}};

     length = Sqrt[(x2-x1)^2+(y2-y1)^2];
     alpha  = Which[ length == 0, 0,
                     length != 0, ArcTan[x2-x1,y2-y1]];

     shorterl = 0.97 * length;
(*   x2p      = x1 + shorterl*Cos[alpha];   *)
     x2p      = 0.95 * 2.0 / 3.0;
(*   y2p      = y1 + shorterl*Sin[alpha];   *)
     y2p      = y1 + (x2p - x1)*Tan[alpha];
     arrowhead = Map[move[#,-alpha,{x2p,y2p}]&, arrow0];

{color,{Line[{{x1,y1},{x2p,y2p}}], Line[arrowhead]}}
];


separateright[x___]:=Module[{f,r,n,m,sx},
		If[Length[Flatten[{x}]]==0, Return[]];
		f = First[x];
		r = Rest[x];
		If[Length[f]==1,
           	   n = First[Union[Flatten[f]]];
		       s1=Join[s1,{StringTake[stringcheck,{n-1,n}]}],
		   m = Union[Flatten[f]];
		       sx=Map[StringTake[stringcheck,{#-1,#}]&,m];
		       s2=Join[s2,sx]
		];
separateright[r]
];


separateleft[x___]:=Module[{f,r,n,m,sx},
		If[Length[Flatten[{x}]]==0, Return[]];
		f = First[x];
		r = Rest[x];
		If[Length[f]==1,
           	   n = First[Union[Flatten[f]]];
		       s1=Join[s1,{StringTake[stringcheck,{n,n+1}]}],
		   m = Union[Flatten[f]];
		       sx=Map[StringTake[stringcheck,{#,#+1}]&,m];
		       s2=Join[s2,sx]
		];
separateleft[r]
];


chars2coords[x___]:=Module[{s1,s2},
	s1=First[ToCharacterCode[First[x]]-96];
	s2=First[ToCharacterCode[Last[x]]-115];
	{leftcoords[[s1]],rightcoords[[s2]]}];


(* MAIN FUNCTION *)

Mappings[arg_, opts___] := Module[
{nl,nr,xl,xr,l,dl,yl,yr,dy,instring,outstring,xstring,mask,finalgraph,frametitle},

     leftsymbols=Characters[StringReplace[ToString[arg[[1]]],
                            {"("->"",")"->"","{"->"",","->"","}"->""," "->""}]];
     rightsymbols=Characters[StringReplace[ToString[arg[[2]]],
                            {"("->"",")"->"","{"->"",","->"","}"->""," "->""}]];
     nl = Length[leftsymbols];
     nr = Length[rightsymbols];

xl = Table[-2/3, {i,nl}];

xr = Table[2/3, {i,nr}];

l = 2; dl = l/nl; yl = Range[1-dl/2, -1+dl/2, -dl];

l = 2; dl = l/nr; yr = Range[1-dl/2, -1+dl/2, -dl];

leftcoords=Transpose[{xl,yl}];

rightcoords=Transpose[{xr,yr}];

points=Point /@ Flatten[{leftcoords,rightcoords}, 1] ;

fs=18; (* font size *)

dx = 0.25; dy = 0;

leftlabels=MapThread[
Text[#1,{-2/3-dx,#2},{-1,0},TextStyle->{FontFamily->"Times", FontSize->fs}]&, 
                     {Take[leftsymbols,nl],yl}];

rightlabels=MapThread[
Text[#1,{2/3+dx,#2},{1,0},TextStyle->{FontFamily->"Times", FontSize->fs}]&, 
                     {Take[rightsymbols,nr],yr}];

labels = Flatten[{leftlabels, rightlabels}];

elements=Flatten[
         {Take[leftsymbols,nl],Take[rightsymbols,nr],"{","}",","}];

(*
instring = Catch[pairstring[nl, nr, arg[[3]]], _Mappings$Error, List];
If[instring==Null, Return[{0, maindiagram[{},opts]}]]
];
*)

instring = arg[[3]];

(* Print[instring]; *)

(* Separate Mappings *)

stringcheck =StringReplace[instring,{"{"->"",","->"","}"->""}];

domain = Take[leftsymbols, nl];

codomain = Take[rightsymbols, nr];

range = Intersection[Characters[stringcheck], codomain];

rightpositions=Map[StringPosition[stringcheck,#]&,
                 Take[rightsymbols,nr]];

leftpositions=Map[StringPosition[stringcheck,#]&,
                 Take[leftsymbols,nl]];

Clear[s1,s2];
s1={}; s2={};
separateleft[leftpositions];
{lgoodstring,lbadstring}={StringJoin[s1],StringJoin[s2]};

(*
Print[lgoodstring,",",lbadstring];
Print[Characters[lgoodstring],Characters[lbadstring]];
*)

Clear[s1,s2];
s1={}; s2={};
separateright[rightpositions];
{rgoodstring,rbadstring}={StringJoin[s1],StringJoin[s2]};

(*
Print[rgoodstring,",",rbadstring];
Print[Characters[rgoodstring],Characters[rbadstring]];
*)

(* Good Mappings *)

lgexpstring = ToExpression[Partition[Characters[lgoodstring],2]];

rgexpstring = Reverse /@ ToExpression[Partition[Characters[rgoodstring],2]];

(*
Print["Good Mappings:"];
Print[lgexpstring];
Print[rgexpstring];
*)

(* Bad Mappings *)

lbexpstring = ToExpression[Partition[Characters[lbadstring],2]];

rbexpstring = Reverse /@ ToExpression[Partition[Characters[rbadstring],2]];

(*
Print["Bad Mappings:"];
Print[lbexpstring];
Print[rbexpstring];
*)

(* Change to Coordinates and Draw the Diagram *)

charstring = Partition[Characters[stringcheck] ,2];

outcoords = Map[chars2coords[#]&, charstring];

(* Print[outcoords]; *)

ls=Map[arrow[#,Red]&, outcoords];

(* Type of Function *)

(*
Print["Domain = ",domain,"\nCodomain = ",codomain,"\nRange = ",range];
*)

xstring = StringReplace[
          StringReplace[instring, {" "->"", "{"->"(", "}"->")"}],
                                  {"("->" ("}];

outstring = "{"<>StringTake[xstring, {3, StringLength[xstring]-1}]<>" }";


mask = 5;

If[(Length[lbexpstring]==0)&&(Length[rbexpstring]==0)&&(nl==nr),
	mask = 4
];

If[(Length[lbexpstring]==0)&&(nl>nr)&&
(nr==Length[Union[Last[Transpose[Partition[Characters[stringcheck],2]]]]]),
	mask = 3
];

If[(Length[lbexpstring]==0)&&(Length[rbexpstring]==0)&&(nl<nr)&&
(nr>=Length[Last[Transpose[Partition[Characters[stringcheck],2]]]]),
	mask = 2
];

If[relation$mask||(Length[lbexpstring]!=0), mask = 1];

out$mask = mask;

If[mask == 4,
   frametitle=StyleForm[
   "\n  This is a 1-1 and ONTO function.\n",
   FontSize->12, FontWeight->"Bold"],
   frametitle=StyleForm[
   "\nThis is "<>getfunctiontype[mask]<>" function.\n",
   FontSize->12, FontWeight->"Bold"]
  ];

finalgraph = maindiagram[ls,PlotLabel->frametitle,opts]
];



(* Epilog *)

SetAttributes[ getfunctiontype, ReadProtected]
SetAttributes[ pairstring,      ReadProtected]
SetAttributes[ arrow,           ReadProtected]
SetAttributes[ separateleft,    ReadProtected]
SetAttributes[ separateright,   ReadProtected]
SetAttributes[ maindiagram,     ReadProtected]
SetAttributes[ chars2coords,    ReadProtected]

SetAttributes[{a,b,c,d,e,f,g,t,u,v,w,x,y,z}, {Locked}]

Protect[Evaluate[mappingsprotected]]

End[]  (*Mappings`Private`*)

SetAttributes[ Mappings,       ReadProtected]
SetAttributes[ Mappings$Error, ReadProtected]
SetAttributes[ MSP$pairstring, ReadProtected]

Protect[Evaluate[m$protected]]

EndPackage[] (* Mappings`*)


