
(* :Title: Operations$1 *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
This package solves exercises with intersection, union, and difference of numerical sets.
*)

(* :Context: SetTheory`Operations$1` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1999. Adapted for MSP, 2002.
*)

(* :Keywords:
	SetTheory, Operations
*)

(* :Source: None. *)

(* :Warning: *)

(* :Mathematica Version: 3.0 or higher *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)

BeginPackage["SetTheory`Operations$1`"]

SetOps$1::usage =
"SetOps$1[R, op] analyses the answer R for the operation op."

MSP$SetOps::usage =
"MSP$SetOps[R, op] picks up the messages from  SetOps$1[R, op]."

Operations1Check


Unprotect[SetOps$1, MSP$SetOps]

Begin["`Private`"]

opsprotected$1 = Unprotect[randomsets, fintersect, funion,
                           testanswer, wronganswer]

cellbkg = RGBColor[1.,0.380001,0.009995] (* CadmiumOrange *) 
evaluation$check = False

(* Auxiliary Functions *)

randomsets[n_] := Module[
     {l,i,y$year,m$month,d$day,h$hour,m$mins,s$secs,
      x$x,tmp$s,out$sets,n$rnd,set$rnd,l$rnd,ind$rnd},
     cnt$ans = 0;
     l    = Table[Random[Integer, {1,9}], {i,1,n}];
     sets = Map[Union[Table[Random[Integer, {1,12}], {i,1,#}]]&, l];
     {y$year,m$month,d$day,h$hour,m$mins,s$secs} = Date[];
     If[(Length[fintersect[sets]]==0)&&(Mod[s$secs, 2]!=0),
        n$rnd = Random[Integer, {1,n}];
        set$rnd = sets[[n$rnd]];
        l$rnd = Length[set$rnd];
        ind$rnd = Union[Table[Random[Integer, {1,l$rnd}], {i, l$rnd}]];
        x$x = set$rnd[[ind$rnd]];
        For[i=1, i<=n, i++, tmp$s = {};
                            tmp$s = Join[sets[[i]], x$x];
                            sets[[i]] = Union[tmp$s]
           ]    
       ];
out$sets=StringDrop[StringJoin[Map[
             (FromCharacterCode[#+64]<>" = "<>ToString[sets[[#]]]<>"\n")&,
              Range[1,n]]], -1]
]

fintersect[{x___}] := Intersection[x]

funion[{x___}] := Union[x]

testanswer[answer__, res__] := Union[answer] === Union[res]

wronganswer[answer__, res__] := Module[{text},
     test = Intersection[answer, res];
     If[ Length[test]==0,
       text = "Your answer: "<>
             ToString[SetTheory`Operations$1`Private`answer]<>
             " is incorrect.",
       text = "Your answer: "<>
             ToString[SetTheory`Operations$1`Private`answer]<>
             " is partially correct.\n"<>
             "The correct part is: "<>
             ToString[SetTheory`Operations$1`Private`test]
       ];
text
]


(* UNION, INTERSECTION, DIFFERENCE *)

MSP$SetOps[R_List, op_String] := Module[{tmp,res},
  tmp = Catch[ SetOps$1[R, op], _Operations1Check, List];
  res = tmp
]


SetOps$1[R_, op_] := Module[{wrans, addcell, addmsg},

     addmsg = Which[op=="difference", "",
                    True, " You may also change the number of sets."];
     If[ cnt$ans>2,
         Throw[cnt$ans, Operations1Check[ "Close this window and try another exercise"<>
                                     " (i.e., generate\nnew sets)."<>addmsg]];
	 Return[False]
       ];
     res    = Which[ op=="union", funion[sets],
                     op=="intersection", fintersect[sets],
                     op=="difference", Complement[funion[sets], sets[[2]]]];
     answer = Union[R];
     If[ testanswer[answer, res],
(* Right answer *)
       cnt$ans=3; 
       Throw[cnt$ans, Operations1Check[ "Your answer: "<>
                   ToString[SetTheory`Operations$1`Private`answer]<>
                   " is correct."]];
       Return[True], 
(* Wrong answer *)
       wrans = wronganswer[answer, res];
       If[ cnt$ans==2,
         addcell = "You failed three times.\n"<>
                   "The correct answer is: "<> 
                   ToString[SetTheory`Operations$1`Private`res];
         cnt$ans++,
         addcell = "Try again"; 
         cnt$ans++
         ]
       ];
     Throw[cnt$ans-1, Operations1Check[ wrans<>"\n\n"<>addcell]];
     Return[False]; 
]


(* EPILOG *)

SetAttributes[ randomsets,  ReadProtected]
SetAttributes[ fintersect,  ReadProtected]
SetAttributes[ funion,      ReadProtected]
SetAttributes[ testanswer,  ReadProtected]
SetAttributes[ wronganswer, ReadProtected]

Protect[Evaluate[opsprotected$1]]

End[]  (*Operations$1`Private`*)

SetAttributes[ SetOps$1,   ReadProtected]
SetAttributes[ MSP$SetOps, ReadProtected]

Unprotect[SetOps$1, MSP$SetOps]

EndPackage[] (* Operations$1`*)


