
(* :Title: Operations$N *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
This package solves exercises with intersection or union of 
more than 2 numerical sets.
*)

(* :Context: SetTheory`Operations$N` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1999. Adapted for MSP, 2002.
*)

(* :Keywords:
	SetTheory, Operations
*)

(* :Source: None. *)

(* :Warning: *)

(* :Mathematica Version: 3.0 or higher *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)

BeginPackage["SetTheory`Operations$N`"]

n$ops::usage =
"n$ops[R] analyses the answer R."

MSP$Ops::usage =
"MSP$Ops[R] picks up the messages from  n$ops[R]."

Not$op

OpsCheck


Unprotect[n$ops, MSP$Ops, Not$op]

Begin["`Private`"]

nopsprotected$1 = Unprotect[randomsets$n, fintersect$n, funion$n,
                           testanswer$n, wronganswer$n, 
                           select$expression, convert$exp]

cellbkg = RGBColor[1.,0.380001,0.009995] (* CadmiumOrange *) 

(* List of exercises *)

list$3 = {"A&(B|C)", "(A\\B)&C", "(A\\B)&(A\\C)",
          "A&B&C", "A|B|C", "(A&B)\\C", "A|(B&C)", "(A\\B)\\C", "A\\(B\\C)"}

list$4 = {"((A&B)\\C)|D", "(A\\B)&(C|D)", "(A|B)\\(C&D)", "(A&B)|(C&D)"}

list$5 = {"A|(B&C)|(D&E)", "(A|B)\\(C&D&E)", "(A&B)|(C&D)|(D&E)",
          "(A&B)|(B&C)|(C&D)|(D&E)"}

(*
list$6 = {"A|B|C(D&E&F)"}

list$7 = {"(A|B|C|D|E)&(F|G)"}

list$8 = {"(A&B\\C)|(!A&D\\E)|(F&G&H)"}

list$9 = {"(A&B\\C)|(D\\E&F)|(!G&H&I)"}
*)

list$exs = {{},{},list$3,list$4,list$5}

(* Auxiliary Functions *)

max$nr$elems = 12

univ$set = Range[1, max$nr$elems]

max$set$length = 9 

randomsets$n[n_Integer] := Module[ {l,i},
     cnt$ans = 0;
     setnumber = n;
     l    = Table[Random[Integer, {1,max$set$length}], {i,1,n}];
     sets = Map[Union[Table[Random[Integer, {1,max$nr$elems}], {i,1,#}]]&, l];
     out$sets=StringDrop[StringJoin[Map[
             (FromCharacterCode[#+64]<>" = "<>ToString[sets[[#]]]<>"\n")&,
              Range[1,n]]], -1]
]

output$format = {"&"->"<SPAN STYLE=\"font-family:Math1\">&#200;</SPAN>",
                 "|"->"<SPAN STYLE=\"font-family:Math1\">&#199;</SPAN>",
                 "!"->"&not;",
                 "_"->"\\"}

select$expression[x___] := Module[{selected$list,i},
     cnt$ans = 0;
     n = setnumber;
     selected$list = list$exs[[n]];
     i = Random[Integer, {1,Length[selected$list]}];
     selected$exp = selected$list[[i]];
     StringReplace[selected$exp, output$format]
]

misc$rules = {"\\012"->"", " "->"", "\("->"\\(",
                              FromCharacterCode[8898]->"&",
                              FromCharacterCode[8899]->"|",
                              FromCharacterCode[172]->"!"}

setminus$ops = {"\\\\"->"& !", 
                  "\\"->"& !", 
                   "_"->"& !"}

basic$rules = {"&"->" \[Intersection] ", "|"->" \[Union] ", 
               "!"->" Not$op @@ "}

Not$op[x__] := Complement[univ$set, {x}]

convert$exp[n_Integer, x_String] := Module[{symbol$rules,s1,s2,s3,ss},

    symbol$rules = Map[("\""<>FromCharacterCode[#+64]<>
               "\"->\"SetTheory`Operations$N`Private`sets[["<>ToString[#]<>
                                                    "]]\"")&, Range[1,n]];
    s1 = StringReplace[ x, misc$rules];
    s2 = StringReplace[s1, setminus$ops];
    s3 = StringReplace[s2, basic$rules];
    ss = ToExpression[StringReplace[s3,ToExpression[symbol$rules]]]
]

fintersect$n[{x___}] := Intersection[x]

funion$n[{x___}] := Union[x]

testanswer$n[answer__, res__] := Union[answer] === Union[res]

wronganswer[answer__, res__] := Module[{text},
     test = Intersection[answer, res];
     If[ Length[test]==0,
       text = "Your answer: "<>
             ToString[SetTheory`Operations$N`Private`answer]<>
             " is incorrect.",
       text = "Your answer: "<>
             ToString[SetTheory`Operations$N`Private`answer]<>
             " is partially correct.\n"<>
             "The correct part is: "<>
             ToString[SetTheory`Operations$N`Private`test]
       ];
text
]


(* GENERAL EXPRESSIONS FOR OPERATIONS WITH SETS *)

MSP$Ops[R_List] := Module[{tmp,res},
  tmp = Catch[ n$ops[R], _OpsCheck, List];
  res = tmp
]


n$ops[R_List] := Module[{wrans, addcell},
     If[ cnt$ans>2,
         Throw[cnt$ans, OpsCheck[ "Close this window and try another exercise"<>
                                     " (i.e., generate\nnew sets and/or expressions)."]];
	 Return[False]
       ];

     res    = convert$exp[setnumber, selected$exp];
     answer = Union[R];
     If[ testanswer$n[answer, res],
(* Right answer *)
   cnt$ans=3; 
       Throw[cnt$ans, OpsCheck[ "Your answer: "<>
                   ToString[SetTheory`Operations$N`Private`answer]<>
                   " is correct."]];
       Return[True], 
(* Wrong answer *)
     wrans = wronganswer[answer, res];
     If[ cnt$ans==2,
         addcell = "You failed three times.\n"<>
                   "The correct answer is: "<> 
                   ToString[SetTheory`Operations$N`Private`res];
         cnt$ans++,
         addcell = "Try again"; 
         cnt$ans++
         ]
       ];
     Throw[cnt$ans-1, OpsCheck[ wrans<>"\n\n"<>addcell]];
     Return[False]; 
]


(* EPILOG *)

SetAttributes[ randomsets$n,      ReadProtected]
SetAttributes[ fintersect$n,      ReadProtected]
SetAttributes[ funion$n,          ReadProtected]
SetAttributes[ testanswer$n,      ReadProtected]
SetAttributes[ wronganswer$n,     ReadProtected]
SetAttributes[ select$expression, ReadProtected]
SetAttributes[ convert$exp,       ReadProtected]

Protect[Evaluate[nopsprotected$1]]

End[]  (*Operations$N`Private`*)

SetAttributes[ n$ops,   ReadProtected]
SetAttributes[ MSP$Ops, ReadProtected]
SetAttributes[ Not$op,  ReadProtected]

Protect[n$ops, MSP$Ops, Not$op]

EndPackage[] (* Operations$N`*)


