
(* :Title: PartsList *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
This is an auxiliary package used by ShowParts[expr].
*)

(* :Context: SetTheory`PartsList` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1998. 
*)

(* :Keywords:
	set theory, elementary logic, reduce logical expressions
*)

(* :Source: The conversion functions for the forms {Principia, Functional, 
            Polish} are modified versions of the conversion functions from 
            the MathSource package 'Logic.m' by Mark J. Watson, Mathematics 
            Department, Hillsdale College, Hillsdale, MI 49242, March 1990.
*)

(* :Warning: *)

(* :Mathematica Version: 3.0 or higher *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)

BeginPackage["SetTheory`PartsList`"];

StringParts::usage = "StringParts[expr] gives a ordered list of elementary
parts of 'expr'.";

V::usage="V[p,q] is the disjunction, 'p or q' or 'p | q'.";

X::usage="X[p,q] is the conjunction, 'p and q' or 'p & q'.";

W::usage="W[p] is the negation, 'not p' or '!p'.";

Functional2Polish::usage="Functional2Polish[x] converts an expression 'x' 
in Functional notation to an equivalent expression in Polish notation.";

Polish2Functional::usage="Polish2Functional[x] converts an expression 'x' 
in Polish notation to an equivalent form in Functional notation.";

Functional2Principia::usage="Functional2Principia[x] converts an expression 
'x' in Functional notation to an equivalent form in Principia notation.";

Principia2Functional::usage="Principia2Functional[x] converts an expression 
'x' in Principia notation to an equivalent form in Functional notation.";

Principia2Polish::usage="Principia2Polish[x] converts an expression 'x' in 
Principia notation to an equivalent form in Polish notation.";

Polish2Principia::usage="Polish2Principia[x] converts an expression 'x' in 
Polish notation to an equivalent form in Principia notation.";

plprotected = Unprotect[StringParts, V, X, W, 
                                 Functional2Polish, Polish2Functional,
                                 Functional2Principia, Principia2Functional,
                                 Principia2Polish, Polish2Principia];

Begin["`Private`"];

Unprotect[xxstringsub, xxparse];

xxstringsub[s_, changes_] := StringJoin @@ (Characters[ToString[s]]/.changes);

list = {};

xxparse["W", y__] := Module[{arg, rest},
                   {arg, rest} = xxparse[y];
                   AppendTo[list, W[arg]];
                   {W[arg], rest}
];

xxparse[x_, y__] := Module[{arg1, arg2, rest},
                  {arg1, rest} = xxparse[y];
                  {arg2, rest} = xxparse @@ rest;
                  AppendTo[list, ToExpression[x][arg1, arg2]];
                  {ToExpression[x][arg1, arg2], rest}
] /; x=="X"||x=="V";
  
xxparse[x_, y___] := {ToExpression[x], List[y]};
   

Functional2Polish[x_] := xxstringsub[x, {"["->"","]"->"",","->""," "->""}];

Polish2Functional[x_] := First[xxparse @@ Characters[ToString[x]]];


Functional2Principia[W[y_]] := StringJoin["!", Functional2Principia[y]];

Functional2Principia[x_[y_, z_]] := StringJoin["(",
                                                Functional2Principia[y],
                                                x/.{V->" | ", X->" & "},
                                                Functional2Principia[z], 
                                                ")"
];

Functional2Principia[x_] := ToString[x];
   
Principia2Functional[x_] := ToExpression[xxstringsub[x,
                             {"!"->"W@", "|"->"~V~", "&"->"~X~"}]];


Principia2Polish[x_] := x//Principia2Functional//Functional2Polish;

Polish2Principia[x_] := x//Polish2Functional//Functional2Principia;


StringParts[x_] := Module[{instring, outstring, s0, s, parts, fcns},
                   list = {};
                   s0 = xxstringsub[x, {"\\\\"->"&!", "\\"->"&!", "_"->"&!"}];
                   instring = xxstringsub[s0, {"A"->"a", "B"->"b", "C"->"c"}];
                   s = Principia2Polish[instring];
                   parts = Polish2Functional[s];
                   fcns  = Map[Functional2Principia[#]&, list];
                   list = {};
                   outstring = Map[
                      xxstringsub[#, {"a"->"A", "b"->"B", "c"->"C"}]&, fcns];
                   outstring
];


 
Protect[xxstringsub, xxparse]

End[]; (* End Private *)

SetAttributes[StringParts,          ReadProtected];
SetAttributes[V,                    ReadProtected];
SetAttributes[X,                    ReadProtected];
SetAttributes[W,                    ReadProtected];
SetAttributes[Functional2Polish,    ReadProtected];
SetAttributes[Polish2Functional,    ReadProtected];
SetAttributes[Functional2Principia, ReadProtected];
SetAttributes[Principia2Functional, ReadProtected];
SetAttributes[Principia2Polish,     ReadProtected];
SetAttributes[Polish2Principia,     ReadProtected];

Protect[Evaluate[plprotected]];

EndPackage[]


