
(* :Title: Power$Sets *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
This package solves exercises with power sets.
*)

(* :Context: SetTheory`Power$Sets` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1999. Adapted for MSP, 2002.
*)

(* :Keywords:
	SetTheory, PowerSets
*)

(* :Source: None. *)

(* :Warning: *)

(* :Mathematica Version: 3.0 or higher *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)

BeginPackage["SetTheory`Power$Sets`"]

MSP$ps$1::usage =
"MSP$ps$1[R] picks up the messages from ps$1[R]."

ps$1::usage =
"ps$1[R] analyses the answer R for power sets."

MSP$psparts$1::usage =
"MSP$psparts$1[R] picks up the messages from psparts$1[R]."

psparts$1::usage =
"psparts$1[R] analyses the answer R for parts of power sets."

PowerSets$msg


Unprotect[ps$1, MSP$ps$1, psparts$1, MSP$psparts$1]

Begin["`Private`"]

psprotected$1 = Unprotect[select$set,ps$f,ps$f$n,
                          select$subset,wronganswer]

cellbkg = RGBColor[1.,0.380001,0.009995] (* CadmiumOrange *) 


(* Auxiliary Functions *)

alpha$set   = Map[FromCharacterCode[#+96]&,Range[1,9]]
numeric$set = Map[FromCharacterCode[#+48]&,Range[1,9]]
total$set   = Union[alpha$set,numeric$set]

select$set[n_] := Module[{selsetout},
cnt$ans = 0;
selected$set = Union[Table[
		  total$set[[Random[Integer,{1,Length[total$set]}]]],
                              {i,n}]];
selsetout = "A = "<>ToString[selected$set]
]

select$subset[x___] := Module[{n$out},
cnt$ans = 0;
n$element$subset = Random[Integer, {0,x}];
n$out = "n = "<>ToString[n$element$subset]
]

ps$f[x_List] := Union[Map[x[[Flatten[
		              Position[Reverse[IntegerDigits[#,2]],1]]]]&,
                                             Range[0,2^Length[x]-1]]]
ps$f[x___]:={}


ps$f$n[n_,x_List] := Module[{powerset,psn},
  powerset=Union[Map[x[[Flatten[Position[Reverse[IntegerDigits[#,2]],1]]]]&,
                     Range[0,2^Length[x]-1]]];
  psn     =Select[powerset, (Length[#]==n)&]
]

ps$f$n[x___]:={}

wronganswer[answer__, res__] := Module[{},
     test = Intersection[answer, res];
     If[ Length[test]==0,
       text = "Your answer: "<>
             ToString[SetTheory`Power$Sets`Private`answer]<>
             " is incorrect.",
       text = "Your answer: "<>
             ToString[SetTheory`Power$Sets`Private`answer]<>
             " is partially correct.\n"<>
             "The correct part is: "<>
             ToString[SetTheory`Power$Sets`Private`test]
       ];
text
]



(* Main Function for Power Sets*)

MSP$ps$1[R_String] := Module[{tmp, res},
  tmp = Catch[ ps$1[R], _PowerSets$msg, List];
  res = tmp
]


ps$1[R_] := Module[{wrans, addcell, RR, tmp, tmp1, tmp2, lcb, rcb},
     If[ cnt$ans>2,
         Throw[cnt$ans, PowerSets$msg[ "Close this window and try another exercise\n"<>
                                     "(i.e., generate a new set)."]];
	 Return[False]
       ];

     res      = Map[StringReplace[ToString[#], {" "->""}]&, 
                                                      ps$f[selected$set]];

     tmp      = Characters[R];
     lcb      = Length[Select[tmp, (#=="{")&]];
     rcb      = Length[Select[tmp, (#=="}")&]];
     If[ lcb != rcb,
         Throw[255, PowerSets$msg[ "Unbalanced curly brackets."]];
	 Return[False]
       ];
     If[ lcb == 1,
         Throw[255, PowerSets$msg[ "You have to use curly brackets to enter a subset:\n"<>
                                   "{ ... }."]];
	 Return[False]
       ];

     tmp1     = Union[Characters[StringReplace[R, {"{"->"","}"->"",","->""}]]];
     tmp2     = Union[Characters[StringReplace[ToString[res], {"{"->"","}"->"",","->""}]]];
     tmp      = Select[tmp1, (!MemberQ[tmp2, #])&];
     If[ Length[tmp] != 0,
         Throw[255, PowerSets$msg[ "Wrong characters/symbols (not in the generated set):\n"<>
                                    StringReplace[ToString[tmp],{"{"->"","}"->""}] ]];
	 Return[False]
       ];

     tmp      = Select[Characters[StringReplace[R, {"},{"->"+"}]], (#=="+")&];
     If[ Length[tmp] != rcb -2,
         Throw[255, PowerSets$msg[ "Missing comma between } and {."]];
	 Return[False]
       ];


     RR       = Map[Union, ToExpression[R]];
     answer   = Union[Map[StringReplace[ToString[#], {" "->""}]&, RR]];

     If[ Length[Intersection[answer, res]]==Length[res],
(* Right answer *)
   cnt$ans=3; 
       Throw[cnt$ans, PowerSets$msg[ "Your answer: "<>
                   ToString[SetTheory`Power$Sets`Private`answer]<>
                   " is correct."]];
       Return[True], 
(* Wrong answer *)
     wrans = wronganswer[answer, res];
     If[ cnt$ans==2,
         addcell = "You failed three times.\n"<>
                   "The correct answer is: "<> 
                   ToString[SetTheory`Power$Sets`Private`res];
         cnt$ans++,
         addcell = "Try again"; 
         cnt$ans++
         ]
       ];
     Throw[cnt$ans-1, PowerSets$msg[ wrans<>"\n\n"<>addcell]];
     Return[False]; 
]



(* Main Function for Power Sets showing Subsets *)

MSP$psparts$1[R_String] := Module[{tmp, res},
  tmp = Catch[ psparts$1[R], _PowerSets$msg, List];
  res = tmp
]


psparts$1[R_] := Module[{wrans, addcell, RR, tmp, tmp1, tmp2, lcb, rcb},
     If[ cnt$ans>2,
         Throw[cnt$ans, PowerSets$msg[ "Close this window and try another exercise\n"<>
                                     "(i.e., generate a new set, or select a new number n)."]];
	 Return[False]
       ];

     res      = Map[StringReplace[ToString[#], {" "->""}]&, 
                                  ps$f$n[n$element$subset,selected$set]];

     tmp      = Characters[R];
     lcb      = Length[Select[tmp, (#=="{")&]];
     rcb      = Length[Select[tmp, (#=="}")&]];
     If[ lcb != rcb,
         Throw[255, PowerSets$msg[ "Unbalanced curly brackets."]];
	 Return[False]
       ];
     If[ lcb == 1,
         Throw[255, PowerSets$msg[ "You have to use curly brackets to enter a subset:\n"<>
                                   "{ ... }."]];
	 Return[False]
       ];

     tmp1     = Union[Characters[StringReplace[R, {"{"->"","}"->"",","->""}]]];
     tmp2     = Union[Characters[StringReplace[ToString[res], {"{"->"","}"->"",","->""}]]];
     tmp      = Select[tmp1, (!MemberQ[tmp2, #])&];
     If[ Length[tmp] != 0,
         Throw[255, PowerSets$msg[ "Wrong characters/symbols (not in the generated set):\n"<>
                                    StringReplace[ToString[tmp],{"{"->"","}"->""}] ]];
	 Return[False]
       ];

     tmp      = Select[Characters[StringReplace[R, {"},{"->"+"}]], (#=="+")&];
     If[ Length[tmp] != rcb -2,
         Throw[255, PowerSets$msg[ "Missing comma between } and {."]];
	 Return[False]
       ];


     RR       = Map[Union, ToExpression[R]];
     answer   = Union[Map[StringReplace[ToString[#], {" "->""}]&, RR]];

     If[ Length[Intersection[answer, res]]==Length[res],
(* Right answer *)
   cnt$ans=3; 
       Throw[cnt$ans, PowerSets$msg[ "Your answer: "<>
                   ToString[SetTheory`Power$Sets`Private`answer]<>
                   " is correct."]];
       Return[True], 
(* Wrong answer *)
     wrans = wronganswer[answer, res];
     If[ cnt$ans==2,
         addcell = "You failed three times.\n"<>
                   "The correct answer is: "<> 
                   ToString[SetTheory`Power$Sets`Private`res];
         cnt$ans++,
         addcell = "Try again"; 
         cnt$ans++
         ]
       ];
     Throw[cnt$ans-1, PowerSets$msg[ wrans<>"\n\n"<>addcell]];
     Return[False]; 
]


(* EPILOG *)

SetAttributes[ select$set,    ReadProtected]
SetAttributes[ select$subset, ReadProtected]
SetAttributes[ ps$f,          ReadProtected]
SetAttributes[ ps$f$n,        ReadProtected]
SetAttributes[ wronganswer,   ReadProtected]

Protect[Evaluate[psprotected$1]]

End[]  (*Power$Sets`Private`*)

SetAttributes[ ps$1,            ReadProtected]
SetAttributes[ MSP$ps$1,        ReadProtected]
SetAttributes[ MSP$psparts$1,   ReadProtected]
SetAttributes[ psparts$1,       ReadProtected]

Protect[ps$1, MSP$ps$1, psparts$1, MSP$psparts$1]

EndPackage[] (* Power$Sets`*)


