
(* :Title: ReduceLogical *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
This is an auxiliary package used by RandomVennDiagram[] to simplify a
logical expression.
*)

(* :Context: SetTheory`ReduceLogical` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1998. 
*)

(* :Keywords:
	set theory, elementary logic, reduce logical expressions
*)

(* :Source: The functions 'convert2function' and 'convert2standard' are
            modified versions of the functions 'PrincipiaToFunctional' and
            'FunctionalToPrincipia' from the MathSource package 'Logic.m' 
            by Mark J. Watson, Mathematics Department, Hillsdale College, 
            Hillsdale, MI 49242, March 1990.
*)

(* :Warning: *)

(* :Mathematica Version: 3.0 or higher *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)

BeginPackage["SetTheory`ReduceLogical`"];

U::usage = "U[a, b] = 'a | b'.";
S::usage = "S[a, b] = 'a & b'.";
H::usage = "H[a] = '!p'.";

convert2standard::usage = "convert2standard[x] converts an expression 'x' in
functional form to an equivalent standard form.";

convert2function::usage = "convert2function[x] converts an expression 'x' in
standard form to an equivalent functional form.";

ReduceExpression::usage = "ReduceExpression[x] reduces a logical expression 
'x' to its simplest form.";

Unprotect[U, S, H, convert2standard, convert2function, ReduceExpression];

Begin["`Private`"];

Share[]

convert2function[x_] := ToExpression[StringReplace[ToString[x],
                                     {"~"->"H@","|"->"~U~","&"->"~S~"}]];

convert2standard[H[x_]]:= StringJoin["~", convert2standard[x]];
convert2standard[x_[y___]] := StringJoin[
                                     "(", convert2standard[First[{y}]],
     Map[ {x /. {U->" | ",S->" & "}, convert2standard[#]}&, Rest[{y}]], 
                                      ")"];
convert2standard[x_] := ToString[x];

nolf = {"\\012"->"", " "->"", FromCharacterCode[8898]->"&",
                              FromCharacterCode[8899]->"|",
                              FromCharacterCode[172]->"!",
        "\("->"\\("};

outputformat = {"&"->FromCharacterCode[8898],
                "|"->FromCharacterCode[8899],
                "!"->FromCharacterCode[172]};

ReduceExpression[x_] := Module[{stin,st0,st1,fst,sst,st2,st3,st4},

                    stin= StringReplace[x, nolf];

                    st0 = StringReplace[stin,
                                     {"\\\\"->"&!", "\\"->"&!", "_"->"&!"}];
                    st1 = StringReplace[ToString[st0],
		                  {"A"->"a", "B"->"b", "C"->"c", "!"->"~"}];
                    fst = convert2function[st1];
                    sst = convert2standard[fst];
                    st2 = StringReplace[sst, 
                                  {"a"->"A", "b"->"B", "c"->"C", "~"->"!"}];
                    st3 = Select[Characters[st2],
                          MemberQ[{"(","A","B","C","!","&","|",")"},#]&];
                    st4 = If[First[st3] == "(", Drop[Drop[st3, 1], -1],
                                                st3];
StringReplace[StringJoin[st4], outputformat] ];

H[H[x___]] := x;
H[S[x_, y_, z_]] := U[H[x], H[y], H[z]];
H[S[x_, y_]] := U[H[x], H[y]];
(*
H[U[x_, y_, z_]] := S[H[x], H[y], H[z]];
H[U[x_, y_]] := S[H[x], H[y]];
*)
U[v___, S[a___, b_, c___], w___, S[x___, b_, y___], z___]:=
U[S[b, U[S[a, c], S[x, y]]], v, w, z];
U[v___, x_, w___, H[x_], z___] := U[v, w, z];
U[v___, x_, w___, x_, z___] := U[v, w, x, z];
U[x_, H[x_]] := U[];
U[x___, U[]] := U[x];
U[x_] := x;

S[x___, U[]] := S[x];
S[x_] := x;

End[]; (* End Private *)

SetAttributes[S, Flatten[{Orderless,ReadProtected,Attributes[Intersection]}]];
SetAttributes[U, Attributes[S]];
SetAttributes[H, ReadProtected];

SetAttributes[convert2standard, ReadProtected];
SetAttributes[convert2function, ReadProtected];
SetAttributes[ReduceExpression, ReadProtected];

Protect[U, S, H, convert2standard, convert2function, ReduceExpression];

EndPackage[]


