
(* :Title: Relations *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
This package displays a mapping between two sets and shows if the
mapping is one-to-one or not.
*)

(* :Context: SetTheory`Relations` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1999. 
*)

(* :Keywords:
	SetTheory, Relation
*)

(* :Source: None. *)

(* :Warning: *)

(* :Mathematica Version: 3.0 *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)
                  
BeginPackage["SetTheory`Relations`"]

MSPRelation::usage = 
"MSPRelation[A,B,R,P] generates the set of all the relations, defined by P, 
bewteen two finite sets (A and B), and check if R belongs to this set."

MSPSymmRel::usage = 
"MSPSymmRel[A,R] generates the set of all the relations defined by the Cartesian 
product  A X A , where  A  is a finite set, and check if R belongs to this set."

Relation

SymmRel

MSP$Relation$sc1

r$protected = Unprotect[MSPRelation, MSPSymmRel, SymmRel, Relation, MSP$Relation$sc1]

Begin["`Private`"]

relationsprotected = Unprotect[pairstring]

(* Colors *)

Blue   = RGBColor[0.,0.,1.] ;
Yellow = RGBColor[1.,1.,0.] ;
Red    = RGBColor[1.,0.,0.];
Green  = RGBColor[0., 1.0, 0.]

cellbkg = RGBColor[1.,0.380001,0.009995] (* CadmiumOrange *) 

Red = RGBColor[1.0, 0., 0.];

ftype = RGBColor[0.529405,0.807794,0.980407];

(* Parameter and Function Definitions *)

nolf = {"\\012"->"", " "->"", "+"->"", "("->"{", ")"->"}"} ;

pairstring[A_, B_, s0_] := Module[
      {ax,bx,elements,nr,nl,s1,s2,sl,sr,s3,sx,nlb,nrb,ncommas,nsymbols},

        ax = Characters[StringReplace[ToString[A],
                        {"{"->"","}"->""," "->"",","->"","\\012"->""}]];
        nl = Length[ax];
        bx = Characters[StringReplace[ToString[B],
                        {"{"->"","}"->""," "->"",","->"","\\012"->""}]];
        nr = Length[bx];
        elements = Union[ax,bx];
	s1 = StringReplace[ToString[s0], nolf];

	sx = Characters[s1];
	nlb = Length[Select[sx, (#=="{")&]];
	nrb = Length[Select[sx, (#=="}")&]];
	If[nlb != nrb, 
	Throw[False, Syntax["Unbalanced parentheses."]];
	];

	ncommas = Length[Select[sx, (#==",")&]];
	If[((Length[sx]-4)/3) != ncommas, 
	Throw[False, Syntax["Too many commas or missing comma(s)."]];
	];

	s2 = Partition[
		Characters[StringReplace[s1,{"{"->"","}"->"",","->""}]],2];
	{sl,sr} = Transpose[s2];
	s3 = Length /@ Map[Union[#]&, s2];
	If[Length[Select[s3, #!=2&]] != 0, 
	       Throw[True, Syntax["Identical elements in one or more pairs."]];
	];

	nsymbols = (Length[sx]-1)/3;
	If[Length[Select[sx, MemberQ[elements,#]&]] != nsymbols, 
	        Throw[True, Syntax["Too many symbols or missing symbol."]];
	];

	If[Length[Intersection[sl,bx]] != 0, 
	        Throw[True, Syntax["Relation(s) defined inside the codomain or\n"<>
                              "inverse relation(s) defined."]];
	];
	If[Length[Intersection[sr,ax]] != 0, 
		Throw[True, Syntax["Relation(s) defined inside the domain."]];
	];
{True, s1}
];


MSP$Relation$sc1[A_, B_, s0_] := Module[{tmp, res},
  tmp = Catch[ pairstring[A, B, s0], _Syntax, List];
  If [ tmp[[1]], 
       res = {True, StringReplace[ToString[s0], nolf]},
       res = tmp
     ]
];



(* MAIN FUNCTIONS *)

Relation[A_,B_,R_,P_] := Module[
{allthepairs,allrelations,Rstring,checkanswer,maxlen,answerlen},

  allthepairs  = Flatten[Outer[List, A, B], 1];
  allrelations = Select[allthepairs, P[#]&];
  maxlen       = Length[allrelations];
  answerlen    = Length[ToExpression[R]];
  checkanswer  = Intersection[allrelations, ToExpression[R]];

If[ Length[checkanswer]==0,
    Throw[False, Relations["The input pair/pairs are wrong."]];
  ];

If[ (Length[checkanswer]==maxlen)&&(Length[checkanswer]==answerlen),
    Throw[True, Relations["The input pair/pairs define the largest possible relation."]];
  ];

If[ (0<Length[checkanswer]<maxlen) && 
      (Length[checkanswer]==answerlen),
     Throw[True, Relations["All the input pair/pairs are correct\n"<>
                           "(not the largest possible relation)."]];
  ];

stanswer = StringReplace[StringDrop[StringDrop[ToString[checkanswer],1],-1], {"{"->"(","}"->")"}];

If[ (Length[checkanswer]>0)&&(Length[checkanswer]!=answerlen),
    Throw[True, Relations["Only the input pair/pairs:\n"<>
                "{"<>SetTheory`Relations`Private`stanswer<>"}"<>
                "\nare correct."]];
  ]
];


MSPRelation[A_,B_,R_,P_] := Module[{tmp, res},
  tmp = Catch[ Relation[A, B, R, P], _Relations, List];
  res = tmp
];


SymmRel[A_, RR_] := Module[ {allthepairs,tt,checkanswer,maxlen,R,answerlen},

  R            = ToExpression[RR];
  allthepairs  = Flatten[Outer[List, A, A], 1];
  maxlen       = Length[allthepairs];
  answerlen    = Length[ToExpression[R]];
  tt           = Select[R, MemberQ[R,Reverse[#]]&];
  checkanswer  = Intersection[R,tt];

If[ Length[checkanswer]==0,
    Throw[False, SymmRels["The input pair/pairs are wrong."]];
  ];

If[ (Length[checkanswer]==maxlen)&&(Length[checkanswer]==answerlen),
    Throw[True, SymmRels["The input pair/pairs define the largest possible relation."]];
  ];

If[ (0<Length[checkanswer]<maxlen) && 
      (Length[checkanswer]==answerlen),
     Throw[True, SymmRels["All the input pair/pairs are correct\n"<>
                           "(not the largest possible relation)."]];
  ];

stanswer = StringReplace[StringDrop[StringDrop[ToString[checkanswer],1],-1], {"{"->"(","}"->")"}];

If[ (Length[checkanswer]>0)&&(Length[checkanswer]!=answerlen),
    Throw[True, SymmRels["Only the input pair/pairs:\n"<>
                "{"<>SetTheory`Relations`Private`stanswer<>"}"<>
                "\nare correct."]];
  ]
];


MSPSymmRel[A_, R_] := Module[{tmp, res},
  tmp = Catch[ SymmRel[A, R], _SymmRels, List];
  res = tmp
];



(* Epilog *)

SetAttributes[ pairstring, ReadProtected]

Protect[Evaluate[relationssprotected]]

End[]  (*Relations`Private`*)

SetAttributes[ MSPRelation,      ReadProtected]
SetAttributes[ MSPSymmRel,       ReadProtected]
SetAttributes[ SymmRel,          ReadProtected]
SetAttributes[ Relation,         ReadProtected]
SetAttributes[ MSP$Relation$sc1, ReadProtected]

Protect[Evaluate[r$protected]]

EndPackage[] (* Relations`*)


