
(* :Title: Set$Complement *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
This package solves exercises with complements of sets.
*)

(* :Context: SetTheory`Set$Complement` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1999. Adapted for MSP, 2002.
*)

(* :Keywords:
	SetTheory, SetComplement
*)

(* :Source: None. *)

(* :Warning: *)

(* :Mathematica Version: 3.0 or higher *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)

BeginPackage["SetTheory`Set$Complement`"]

sc$1::usage =
"sc$1[R] analyses the answer R."

MSP$SetCo::usage =
"MSP$SetCo[R] picks up the messages from  sc$1[R]."

SetCoCheck


Unprotect[sc$1, MSP$SetCo]

Begin["`Private`"]

scnprotected$1 = Unprotect[randomsets, testanswer, wronganswer,
                           select$exp, Lc]

cellbkg = RGBColor[1.,0.380001,0.009995] (* CadmiumOrange *) 


(* Auxiliary Functions *)

max$length = 9

universal$set = Range[1,max$length]

expressions$list = {
  "<i>A</i><sup>c</sup>",
  "(<i>A</i> <SPAN STYLE=\"font-family:Math1\">&#199;</SPAN> <i>B</i>)<sup>c</sup>",
  "(<i>A</i><sup>c</sup> <SPAN STYLE=\"font-family:Math1\">&#199;</SPAN> <i>B</i>)<sup>c</sup>",
  "(<i>A</i><sup>c</sup> <SPAN STYLE=\"font-family:Math1\">&#200;</SPAN> <i>B</i>)<sup>c</sup>",
  "(<i>A</i><sup>c</sup> <SPAN STYLE=\"font-family:Math1\">&#199;</SPAN> <i>B</i><sup>c</sup>)<sup>c</sup>"
                   }

internal$list = {"!A", "!(A|B)", "!(!A|B)", "!(!A&B)", "!(!A|!B)"}

select$exp[x___] := Module[{sel$exp},
     cnt$ans = 0;
     n$exp = Random[Integer, {1,Length[expressions$list]}];
     sel$exp = expressions$list[[n$exp]];
     int$exp = internal$list[[n$exp]];
     n$sets = Length[Union[Characters[StringReplace[int$exp, 
                           {"("->"",")"->"","!"->"","|"->"","&"->""}]]]];
     sel$exp
]

randomsets[y___] := Module[{l,i,out$sets},
     cnt$ans = 0;
     n    = n$sets;
     l    = Table[Random[Integer, {1,max$length}], {i,1,n}];
     sets = Map[Union[Table[Random[Integer, {1,max$length}], {i,1,#}]]&, l];
out$sets=StringDrop[StringJoin[Map[
             (FromCharacterCode[#+64]<>" = "<>ToString[sets[[#]]]<>"\n")&,
              Range[1,n]]], -1]
]

Lc[x__] := Complement[universal$set, {x}]

testanswer[answer__, res__] := Union[answer] === Union[res]

wronganswer[answer__, res__] := Module[{text},
     test = Intersection[answer, res];
     If[ Length[test]==0,
       text = "Your answer: "<>
             ToString[SetTheory`Set$Complement`Private`answer]<>
             " is incorrect.",
       text = "Your answer: "<>
             ToString[SetTheory`Set$Complement`Private`answer]<>
             " is partially correct.\n"<>
             "The correct part is: "<>
             ToString[SetTheory`Set$Complement`Private`test]
       ];
text
]


(* Main Functions *)

MSP$SetCo[R_List] := Module[{tmp,res},
  tmp = Catch[ sc$1[R], _SetCoCheck, List];
  res = tmp
]


sc$1[R_List] := Module[{res$1, wrans, addcell},
     If[ cnt$ans>2,
         Throw[cnt$ans, SetCoCheck[ "Close this window and try another exercise"<>
                                     " (i.e., generate\nnew expressions and/or sets)."]];
	 Return[False]
       ];

     res$1  = StringReplace[int$exp, 
                {"&"->" \[Intersection] ", "|"->" \[Union] ", 
                 "!"->"SetTheory`Set$Complement`Private`Lc @@ ",
                 "A"->"SetTheory`Set$Complement`Private`sets[[1]]",     
                 "B"->"SetTheory`Set$Complement`Private`sets[[2]]",     
                 "C"->"SetTheory`Set$Complement`Private`sets[[3]]"}];
     res    = ToExpression[res$1];     
     answer = Union[R] ;

     If[ testanswer[answer, res],
(* Right answer *)
       cnt$ans=3; 
       Throw[cnt$ans, SetCoCheck[ "Your answer: "<>
                   ToString[SetTheory`Set$Complement`Private`answer]<>
                   " is correct."]];
       Return[True], 
(* Wrong answer *)
     wrans = wronganswer[answer, res];
     If[ cnt$ans==2,
         addcell = "You failed three times.\n"<>
                   "The correct answer is: "<> 
                   ToString[SetTheory`Set$Complement`Private`res];
         cnt$ans++,
         addcell = "Try again"; 
         cnt$ans++
         ]
       ];
     Throw[cnt$ans-1, SetCoCheck[ wrans<>"\n\n"<>addcell]];
     Return[False]; 
]


(* EPILOG *)

SetAttributes[ randomsets,  ReadProtected]
SetAttributes[ testanswer,  ReadProtected]
SetAttributes[ wronganswer, ReadProtected]
SetAttributes[ select$exp,  ReadProtected]
SetAttributes[ Lc,          ReadProtected]

Protect[Evaluate[scnprotected$1]]

End[]  (*Set$Complement`Private`*)

SetAttributes[ sc$1,      ReadProtected]
SetAttributes[ MSP$SetCo, ReadProtected]

Protect[sc$1, MSP$SetCo]

EndPackage[] (* Set$Complement`*)


